@@ -47,7 +47,7 @@ CLIENT_NAME = 'nova'
 class NovaClientPlugin(microversion_mixin.MicroversionMixin,
                        client_plugin.ClientPlugin):
 
-    deferred_server_statuses = ['BUILD',
+    deferred_server_statuses = {'BUILD',
                                 'HARD_REBOOT',
                                 'PASSWORD',
                                 'REBOOT',
@@ -56,7 +56,7 @@ class NovaClientPlugin(microversion_mixin.MicroversionMixin,
                                 'REVERT_RESIZE',
                                 'SHUTOFF',
                                 'SUSPENDED',
-                                'VERIFY_RESIZE']
+                                'VERIFY_RESIZE'}
 
     exceptions_module = exceptions
 
@@ -281,20 +281,15 @@ class NovaClientPlugin(microversion_mixin.MicroversionMixin,
 
         return flavor
 
-    def get_host(self, host_name):
-        """Get the host id specified by name.
+    def get_host(self, hypervisor_hostname):
+        """Gets list of matching hypervisors by specified name.
 
-        :param host_name: the name of host to find
-        :returns: the list of match hosts
-        :raises exception.EntityNotFound:
+        :param hypervisor_hostname: the name of host to find
+        :returns: list of matching hypervisor hosts
+        :raises nova client exceptions.NotFound:
         """
 
-        host_list = self.client().hosts.list()
-        for host in host_list:
-            if host.host_name == host_name and host.service == self.COMPUTE:
-                return host
-
-        raise exception.EntityNotFound(entity='Host', name=host_name)
+        return self.client().hypervisors.search(hypervisor_hostname)
 
     def get_keypair(self, key_name):
         """Get the public key specified by :key_name:
@@ -601,7 +596,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
 
     def meta_serialize(self, metadata):
         """Serialize non-string metadata values before sending them to Nova."""
-        if not isinstance(metadata, collections.Mapping):
+        if not isinstance(metadata, collections.abc.Mapping):
             raise exception.StackValidationFailed(message=_(
                 "nova server metadata needs to be a Map."))
 
@@ -652,7 +647,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         """
         nc = self.client
 
-        class ConsoleUrls(collections.Mapping):
+        class ConsoleUrls(collections.abc.Mapping):
             def __init__(self, server):
                 self.console_method = server.get_console_url
                 self.support_console_types = ['novnc', 'xvpvnc',
@@ -693,7 +688,9 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                 volume_id=volume_id,
                 device=device)
         except Exception as ex:
-            if self.is_client_exception(ex):
+            if self.is_conflict(ex):
+                return False
+            elif self.is_client_exception(ex):
                 raise exception.Error(_(
                     "Failed to attach volume %(vol)s to server %(srv)s "
                     "- %(err)s") % {'vol': volume_id,
@@ -708,12 +705,15 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         try:
             self.client().volumes.delete_server_volume(server_id, attach_id)
         except Exception as ex:
-            if not (self.is_not_found(ex)
-                    or self.is_bad_request(ex)):
+            if self.is_conflict(ex):
+                return False
+            elif not (self.is_not_found(ex)
+                      or self.is_bad_request(ex)):
                 raise exception.Error(
                     _("Could not detach attachment %(att)s "
                       "from server %(srv)s.") % {'srv': server_id,
                                                  'att': attach_id})
+        return True
 
     def check_detach_volume_complete(self, server_id, attach_id):
         """Check that nova server lost attachment.
@@ -867,4 +867,6 @@ class FlavorConstraint(NovaBaseConstraint):
 
 class HostConstraint(NovaBaseConstraint):
 
+    expected_exceptions = (exceptions.NotFound,)
+
     resource_getter_name = 'get_host'
